@@ -32,11 +32,10 @@
 class ClickLineEdit;
 class QTimer;
 class KActionCollection;
-class KDirOperator;
 class KFileItem;
 class KFileView;
 class KURLComboBox;
-
+class Medium;
 
 //Hi! I think we ripped this from Kate, since then it's been modified somewhat
 
@@ -53,24 +52,28 @@ class FileBrowser : public QVBox
 {
     Q_OBJECT
 
-    enum MenuId { MakePlaylist, SavePlaylist, MediaDevice, AppendToPlaylist, SelectAllFiles, BurnCd };
+    enum MenuId { MakePlaylist, SavePlaylist, MediaDevice, AppendToPlaylist, QueueTrack, QueueTracks, SelectAllFiles, BurnCd, MoveToCollection, CopyToCollection, OrganizeFiles, EditTags };
 
 public:
-    FileBrowser( const char *name = 0 );
+    FileBrowser( const char *name = 0, Medium *medium = 0 );
    ~FileBrowser();
 
     KURL url() const { return m_dir->url(); }
+
 public slots:
-    void setUrl( const KURL &url ) { m_dir->setFocus(); m_dir->setURL( url, true ); }
-    void setUrl( const QString &url ) { setUrl( KURL(url) ); }
+    void setUrl( const KURL &url );
+    void setUrl( const QString &url );
     void setFilter( const QString& );
+    void dropped( const KFileItem*, QDropEvent*, const KURL::List& );
 
 private slots:
-    void urlChanged( const KURL& );
     void activate( const KFileItem* );
     void contextMenuActivated( int );
+    void gotoCurrentFolder();
     void prepareContextMenu();
+    void selectAll();
     void slotViewChanged( KFileView* );
+    void urlChanged( const KURL& );
 
 private:
     KURL::List selectedItems();
@@ -79,6 +82,7 @@ private:
     KURLComboBox  *m_combo;
     KDirOperator  *m_dir;
     ClickLineEdit *m_filter;
+    Medium        *m_medium;
 };
 
 
