@@ -1,6 +1,8 @@
 /***************************************************************************
- *   Copyright (C) 2004-2005 by Mark Kretschmann <markey@web.de>           *
+ *   Copyright (C) 2004-2006 by Mark Kretschmann <markey@web.de>           *
  *                      2005 by Seb Ruiz <me@sebruiz.net>                  *
+ *                      2006 by Alexandre Oliveira <aleprj@gmail.com>      *
+ *                      2006 by Martin Ellis <martin.ellis@kdemail.net>    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -15,17 +17,20 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
  ***************************************************************************/
 
 #define DEBUG_PREFIX "ScriptManager"
 
 #include "amarok.h"
+#include "amarokconfig.h"
+#include "contextbrowser.h"
 #include "debug.h"
 #include "enginecontroller.h"
 #include "metabundle.h"
 #include "scriptmanager.h"
 #include "scriptmanagerbase.h"
+#include "statusbar.h"
 
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -35,6 +40,7 @@
 #include <qfileinfo.h>
 #include <qfont.h>
 #include <qlabel.h>
+#include <qtextcodec.h>
 #include <qtimer.h>
 
 #include <kaboutdialog.h>
@@ -47,6 +53,7 @@
 #include <kmessagebox.h>
 #include <kpopupmenu.h>
 #include <kprocio.h>
+#include <kprotocolmanager.h>
 #include <kpushbutton.h>
 #include <krun.h>
 #include <kstandarddirs.h>
@@ -59,32 +66,44 @@
 #include <knewstuff/knewstuff.h>      // "
 #include <knewstuff/provider.h>       // "
 
-namespace amaroK {
+
+namespace Amarok {
     void closeOpenFiles(int out, int in, int err) {
         for(int i = sysconf(_SC_OPEN_MAX) - 1; i > 2; i--)
             if(i!=out && i!=in && i!=err)
                 close(i);
     }
+
+     /**
+    * This constructor is needed so that the correct codec is used. KProcIO defaults
+    * to latin1, while the scanner uses UTF-8.
+    */
+    ProcIO::ProcIO() : KProcIO( QTextCodec::codecForName( "UTF-8" ) ) {}
+
+    QString
+    proxyForUrl(const QString& url)
+    {
+        KURL kurl( url );
+
+        QString proxy;
+
+        if ( KProtocolManager::proxyForURL( kurl ) != 
+                QString::fromLatin1( "DIRECT" ) ) {
+            KProtocolManager::slaveProtocol ( kurl, proxy );
+        }
+
+        return proxy;
+    }
+
+    QString
+    proxyForProtocol(const QString& protocol)
+    {
+        return KProtocolManager::proxyFor( protocol );
+    }
+
+
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// class AmaroKProcIO
-////////////////////////////////////////////////////////////////////////////////
-/** Due to xine-lib, we have to make KProcess close all fds, otherwise we get "device is busy" messages
-  * Used by AmaroKProcIO and AmaroKProcess, exploiting commSetupDoneC(), a virtual method that
-  * happens to be called in the forked process
-  * See bug #103750 for more information.
-  */
-class AmaroKProcIO : public KProcIO {
-    public:
-    virtual int commSetupDoneC() {
-        const int i = KProcIO::commSetupDoneC();
-        amaroK::closeOpenFiles(KProcIO::out[0],KProcIO::in[0],KProcIO::err[0]);
-        return i;
-    };
-};
-
-
 ////////////////////////////////////////////////////////////////////////////////
 // class AmarokScriptNewStuff
 ////////////////////////////////////////////////////////////////////////////////
@@ -116,7 +135,7 @@ ScriptManager* ScriptManager::s_instance = 0;
 
 
 ScriptManager::ScriptManager( QWidget *parent, const char *name )
-        : KDialogBase( parent, name, false, 0, 0, Ok, false )
+        : KDialogBase( parent, name, false, QString::null, Close, Close, true )
         , EngineObserver( EngineController::instance() )
         , m_gui( new ScriptManagerBase( this ) )
 {
@@ -132,12 +151,37 @@ ScriptManager::ScriptManager( QWidget *parent, const char *name )
     KWin::setState( winId(), NET::SkipTaskbar );
 
     setMainWidget( m_gui );
+    m_gui->listView->setRootIsDecorated( true );
     m_gui->listView->setFullWidth( true );
     m_gui->listView->setShowSortIndicator( true );
 
+
+    /// Category items
+    m_generalCategory    = new KListViewItem( m_gui->listView, i18n( "General" ) );
+    m_lyricsCategory     = new KListViewItem( m_gui->listView, i18n( "Lyrics" ) );
+    m_scoreCategory      = new KListViewItem( m_gui->listView, i18n( "Score" ) );
+    m_transcodeCategory  = new KListViewItem( m_gui->listView, i18n( "Transcoding" ) );
+
+    m_generalCategory  ->setSelectable( false );
+    m_lyricsCategory   ->setSelectable( false );
+    m_scoreCategory    ->setSelectable( false );
+    m_transcodeCategory->setSelectable( false );
+
+    m_generalCategory  ->setPixmap( 0, SmallIcon( Amarok::icon( "files" ) ) );
+    m_lyricsCategory   ->setPixmap( 0, SmallIcon( Amarok::icon( "files" ) ) );
+    m_scoreCategory    ->setPixmap( 0, SmallIcon( Amarok::icon( "files" ) ) );
+    m_transcodeCategory->setPixmap( 0, SmallIcon( Amarok::icon( "files" ) ) );
+
+    // Restore the open/closed state of the category items
+    KConfig* const config = Amarok::config( "ScriptManager" );
+    m_generalCategory  ->setOpen( config->readBoolEntry( "General category open" ) );
+    m_lyricsCategory   ->setOpen( config->readBoolEntry( "Lyrics category open" ) );
+    m_scoreCategory    ->setOpen( config->readBoolEntry( "Score category State" ) );
+    m_transcodeCategory->setOpen( config->readBoolEntry( "Transcode category open" ) );
+
     connect( m_gui->listView, SIGNAL( currentChanged( QListViewItem* ) ), SLOT( slotCurrentChanged( QListViewItem* ) ) );
     connect( m_gui->listView, SIGNAL( doubleClicked ( QListViewItem*, const QPoint&, int ) ), SLOT( slotRunScript() ) );
-    connect( m_gui->listView, SIGNAL( rightButtonPressed ( QListViewItem*, const QPoint&, int ) ), SLOT( slotShowContextMenu( QListViewItem*, const QPoint& ) ) );
+    connect( m_gui->listView, SIGNAL( contextMenuRequested ( QListViewItem*, const QPoint&, int ) ), SLOT( slotShowContextMenu( QListViewItem*, const QPoint& ) ) );
 
     connect( m_gui->installButton,   SIGNAL( clicked() ), SLOT( slotInstallScript() ) );
     connect( m_gui->retrieveButton,  SIGNAL( clicked() ), SLOT( slotRetrieveScript() ) );
@@ -147,18 +191,20 @@ ScriptManager::ScriptManager( QWidget *parent, const char *name )
     connect( m_gui->configureButton, SIGNAL( clicked() ), SLOT( slotConfigureScript() ) );
     connect( m_gui->aboutButton,     SIGNAL( clicked() ), SLOT( slotAboutScript() ) );
 
-    m_gui->installButton  ->setIconSet( SmallIconSet( "fileopen" ) );
-    m_gui->retrieveButton ->setIconSet( SmallIconSet( "khtml_kget" ) );
-    m_gui->uninstallButton->setIconSet( SmallIconSet( "remove" ) );
-    m_gui->runButton      ->setIconSet( SmallIconSet( "player_play" ) );
-    m_gui->stopButton     ->setIconSet( SmallIconSet( "player_stop" ) );
-    m_gui->configureButton->setIconSet( SmallIconSet( "configure" ) );
-    m_gui->aboutButton    ->setIconSet( SmallIconSet( "help" ) );
+    m_gui->installButton  ->setIconSet( SmallIconSet( Amarok::icon( "files" ) ) );
+    m_gui->retrieveButton ->setIconSet( SmallIconSet( Amarok::icon( "download" ) ) );
+    m_gui->uninstallButton->setIconSet( SmallIconSet( Amarok::icon( "remove" ) ) );
+    m_gui->runButton      ->setIconSet( SmallIconSet( Amarok::icon( "play" ) ) );
+    m_gui->stopButton     ->setIconSet( SmallIconSet( Amarok::icon( "stop" ) ) );
+    m_gui->configureButton->setIconSet( SmallIconSet( Amarok::icon( "configure" ) ) );
+    m_gui->aboutButton    ->setIconSet( SmallIconSet( Amarok::icon( "info" ) ) );
 
     QSize sz = sizeHint();
     setMinimumSize( kMax( 350, sz.width() ), kMax( 250, sz.height() ) );
     resize( sizeHint() );
 
+    connect( this, SIGNAL(lyricsScriptChanged()), ContextBrowser::instance(), SLOT( lyricsScriptChanged() ) );
+
     // Delay this call via eventloop, because it's a bit slow and would block
     QTimer::singleShot( 0, this, SLOT( findScripts() ) );
 }
@@ -168,24 +214,25 @@ ScriptManager::~ScriptManager()
 {
     DEBUG_BLOCK
 
-    debug() << "Terminating running scripts.\n";
-
     QStringList runningScripts;
     ScriptMap::Iterator it;
     ScriptMap::Iterator end( m_scripts.end() );
     for( it = m_scripts.begin(); it != end; ++it ) {
         if( it.data().process ) {
-            it.data().process->kill(); // Sends SIGTERM
-            it.data().process->detach();
-            delete it.data().process;
+            terminateProcess( &it.data().process );
             runningScripts << it.key();
         }
     }
 
     // Save config
-    KConfig* const config = amaroK::config( "ScriptManager" );
+    KConfig* const config = Amarok::config( "ScriptManager" );
     config->writeEntry( "Running Scripts", runningScripts );
-    config->writeEntry( "Auto Run", m_gui->checkBox_autoRun->isChecked() );
+
+    // Save the open/closed state of the category items
+    config->writeEntry( "General category open", m_generalCategory->isOpen() );
+    config->writeEntry( "Lyrics category open", m_lyricsCategory->isOpen() );
+    config->writeEntry( "Score category open", m_scoreCategory->isOpen() );
+    config->writeEntry( "Transcode category open", m_transcodeCategory->isOpen() );
 
     s_instance = 0;
 }
@@ -196,13 +243,13 @@ ScriptManager::~ScriptManager()
 ////////////////////////////////////////////////////////////////////////////////
 
 bool
-ScriptManager::runScript( const QString& name )
+ScriptManager::runScript( const QString& name, bool silent )
 {
     if( !m_scripts.contains( name ) )
         return false;
 
     m_gui->listView->setCurrentItem( m_scripts[name].li );
-    return slotRunScript();
+    return slotRunScript( silent );
 }
 
 
@@ -223,9 +270,7 @@ QStringList
 ScriptManager::listRunningScripts()
 {
     QStringList runningScripts;
-    ScriptMap::ConstIterator it;
-    ScriptMap::ConstIterator end(m_scripts.end() );
-    for( it = m_scripts.begin(); it != end; ++it )
+    foreachType( ScriptMap, m_scripts )
         if( it.data().process )
             runningScripts << it.key();
 
@@ -240,6 +285,68 @@ ScriptManager::customMenuClicked( const QString& message )
 }
 
 
+QString
+ScriptManager::specForScript( const QString& name )
+{
+    if( !m_scripts.contains( name ) )
+        return QString();
+    QFileInfo info( m_scripts[name].url.path() );
+    const QString specPath = info.dirPath() + '/' + info.baseName( true ) + ".spec";
+
+    return specPath;
+}
+
+
+void
+ScriptManager::notifyFetchLyrics( const QString& artist, const QString& title )
+{
+    const QString args = KURL::encode_string( artist ) + ' ' + KURL::encode_string( title );
+    notifyScripts( "fetchLyrics " + args );
+}
+
+
+void
+ScriptManager::notifyFetchLyricsByUrl( const QString& url )
+{
+    notifyScripts( "fetchLyricsByUrl " + url );
+}
+
+
+void ScriptManager::notifyTranscode( const QString& srcUrl, const QString& filetype )
+{
+    notifyScripts( "transcode " + srcUrl + ' ' + filetype );
+}
+
+
+void
+ScriptManager::notifyPlaylistChange( const QString& change)
+{
+   notifyScripts( "playlistChange: " + change );
+}
+
+
+void
+ScriptManager::requestNewScore( const QString &url, double prevscore, int playcount, int length, float percentage, const QString &reason )
+{
+    const QString script = ensureScoreScriptRunning();
+    if( script.isNull() )
+    {
+        Amarok::StatusBar::instance()->longMessage(
+            i18n( "No score scripts were found, or none of them worked. Automatic scoring will be disabled. Sorry." ),
+            KDE::StatusBar::Sorry );
+        return;
+    }
+
+    m_scripts[script].process->writeStdin(
+        QString( "requestNewScore %6 %1 %2 %3 %4 %5" )
+        .arg( prevscore )
+        .arg( playcount )
+        .arg( length )
+        .arg( percentage )
+        .arg( reason )
+        .arg( KURL::encode_string( url ) ) ); //last because it might have %s
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // private slots
 ////////////////////////////////////////////////////////////////////////////////
@@ -247,36 +354,27 @@ ScriptManager::customMenuClicked( const QString& message )
 void
 ScriptManager::findScripts() //SLOT
 {
-    DEBUG_BLOCK
-
     const QStringList allFiles = kapp->dirs()->findAllResources( "data", "amarok/scripts/*", true );
 
     // Add found scripts to listview:
-
-    QStringList::ConstIterator it;
-    QStringList::ConstIterator end( allFiles.end() );
-    for( it = allFiles.begin(); it != end; ++it )
-        if( QFileInfo( *it ).isExecutable() )
-            loadScript( *it );
+    {
+        foreach( allFiles )
+            if( QFileInfo( *it ).isExecutable() )
+                loadScript( *it );
+    }
 
     // Handle auto-run:
 
-    KConfig* const config = amaroK::config( "ScriptManager" );
-    const bool autoRun = config->readBoolEntry( "Auto Run", false );
-    m_gui->checkBox_autoRun->setChecked( autoRun );
+    KConfig* const config = Amarok::config( "ScriptManager" );
+    const QStringList runningScripts = config->readListEntry( "Running Scripts" );
 
-    if( autoRun ) {
-        const QStringList runningScripts = config->readListEntry( "Running Scripts" );
-
-        QStringList::ConstIterator it;
-        QStringList::ConstIterator end( runningScripts.end() );
-        for( it = runningScripts.begin(); it != end; ++it ) {
+    {
+        foreach( runningScripts )
             if( m_scripts.contains( *it ) ) {
                 debug() << "Auto-running script: " << *it << endl;
                 m_gui->listView->setCurrentItem( m_scripts[*it].li );
                 slotRunScript();
             }
-        }
     }
 
     m_gui->listView->setCurrentItem( m_gui->listView->firstChild() );
@@ -287,7 +385,12 @@ ScriptManager::findScripts() //SLOT
 void
 ScriptManager::slotCurrentChanged( QListViewItem* item )
 {
-    if( item ) {
+    const bool isCategory = item == m_generalCategory ||
+                            item == m_lyricsCategory ||
+                            item == m_scoreCategory ||
+                            item == m_transcodeCategory;
+
+    if( item && !isCategory ) {
         const QString name = item->text( 0 );
         m_gui->uninstallButton->setEnabled( true );
         m_gui->runButton->setEnabled( !m_scripts[name].process );
@@ -308,8 +411,6 @@ ScriptManager::slotCurrentChanged( QListViewItem* item )
 bool
 ScriptManager::slotInstallScript( const QString& path )
 {
-    DEBUG_BLOCK
-
     QString _path = path;
 
     if( path.isNull() ) {
@@ -318,7 +419,7 @@ ScriptManager::slotInstallScript( const QString& path )
             + i18n( "Script Packages (*.amarokscript.tar, *.amarokscript.tar.bz2, *.amarokscript.tar.gz)" )
             , this
             , i18n( "Select Script Package" ) );
-        if( _path == QString::null ) return false;
+        if( _path.isNull() ) return false;
     }
 
     KTar archive( _path );
@@ -327,7 +428,7 @@ ScriptManager::slotInstallScript( const QString& path )
         return false;
     }
 
-    QString destination = amaroK::saveLocation( "scripts/" );
+    QString destination = Amarok::saveLocation( "scripts/" );
     const KArchiveDirectory* const archiveDir = archive.directory();
 
     // Prevent installing a script that's already installed
@@ -364,15 +465,13 @@ ScriptManager::recurseInstall( const KArchiveDirectory* archiveDir, const QStrin
 {
     const QStringList entries = archiveDir->entries();
 
-    QStringList::ConstIterator it;
-    QStringList::ConstIterator end( entries.end() );
-    for( it = entries.begin(); it != end; ++it ) {
+    foreach( entries ) {
         const QString entry = *it;
         const KArchiveEntry* const archEntry = archiveDir->entry( entry );
 
         if( archEntry->isDirectory() ) {
-            KArchiveDirectory* const dir = (KArchiveDirectory*) archEntry;
-            recurseInstall( dir, destination + entry + "/" );
+            const KArchiveDirectory* const dir = static_cast<const KArchiveDirectory*>( archEntry );
+            recurseInstall( dir, destination + entry + '/' );
         }
         else {
             ::chmod( QFile::encodeName( destination + entry ), archEntry->permissions() );
@@ -392,7 +491,7 @@ ScriptManager::slotRetrieveScript()
     // Delete KNewStuff's configuration entries. These entries reflect which scripts
     // are already installed. As we cannot yet keep them in sync after uninstalling
     // scripts, we deactivate the check marks entirely.
-    amaroK::config()->deleteGroup( "KNewStuffStatus" );
+    Amarok::config()->deleteGroup( "KNewStuffStatus" );
 
     // we need this because KNewStuffGeneric's install function isn't clever enough
     AmarokScriptNewStuff *kns = new AmarokScriptNewStuff( "amarok/script", this );
@@ -411,8 +510,6 @@ ScriptManager::slotRetrieveScript()
 void
 ScriptManager::slotUninstallScript()
 {
-    DEBUG_BLOCK
-
     const QString name = m_gui->listView->currentItem()->text( 0 );
 
     if( KMessageBox::warningContinueCancel( 0, i18n( "Are you sure you want to uninstall the script '%1'?" ).arg( name ), i18n("Uninstall Script"), i18n("Uninstall") ) == KMessageBox::Cancel )
@@ -421,71 +518,111 @@ ScriptManager::slotUninstallScript()
     if( m_scripts.find( name ) == m_scripts.end() )
         return;
 
-    const QString directory = m_scripts[name].url.directory();
+    KURL scriptDirURL( m_scripts[name].url.upURL() );
 
-    // Delete directory recursively
-    const KURL url = KURL::fromPathOrURL( directory );
-    if( !KIO::NetAccess::del( url, 0 ) ) {
-        KMessageBox::sorry( 0, i18n( "<p>Could not uninstall this script.</p><p>The ScriptManager can only uninstall scripts which have been installed as packages.</p>" ) );
+    // find if the script is installed in the global or local scripts directory
+    KURL scriptsDirURL;
+    QStringList dirs = KGlobal::dirs()->findDirs( "data", "amarok/scripts/" );
+    for ( QStringList::Iterator it = dirs.begin(); it != dirs.end(); ++it ) {
+        scriptsDirURL = KURL::fromPathOrURL( *it );
+        if ( scriptsDirURL.isParentOf( scriptDirURL ) )
+            break;
+    }
+
+    // find the begining of this script directory tree
+    KURL scriptDirUpURL = scriptDirURL.upURL();
+    while ( ! scriptsDirURL.equals( scriptDirUpURL, true ) && scriptsDirURL.isParentOf( scriptDirUpURL ) ) {
+        scriptDirURL = scriptDirUpURL;
+        scriptDirUpURL = scriptDirURL.upURL();
+    }
+
+    // Delete script directory recursively
+    if( !KIO::NetAccess::del( scriptDirURL, 0 ) ) {
+        KMessageBox::sorry( 0, i18n( "<p>Could not uninstall this script.</p><p>The ScriptManager can only uninstall scripts which have been installed as packages.</p>" ) ); // only true when not running as root (which is reasonable)
         return;
     }
 
-    // Find all scripts that were in the uninstalled folder
     QStringList keys;
-    ScriptMap::Iterator it;
-    ScriptMap::Iterator end( m_scripts.end() );
-    for( it = m_scripts.begin(); it != end; ++it )
-        if( it.data().url.directory() == directory )
-            keys << it.key();
 
-    // Kill script processes, remove entries from script list
-    QStringList::Iterator itKeys;
-    for( itKeys = keys.begin(); itKeys != keys.end(); ++itKeys ) {
-        delete m_scripts[*itKeys].li;
-        if( m_scripts[*itKeys].process ) {
-            // Terminate script process (with SIGTERM)
-            m_scripts[*itKeys].process->kill();
-            m_scripts[*itKeys].process->detach();
-            delete m_scripts[*itKeys].process;
+    // Find all scripts that were in the uninstalled directory
+    {
+        foreachType( ScriptMap, m_scripts )
+            if( scriptDirURL.isParentOf( it.data().url ) )
+                keys << it.key();
+    }
+
+    // Terminate script processes, remove entries from script list
+    {
+        foreach( keys ) {
+            delete m_scripts[*it].li;
+            terminateProcess( &m_scripts[*it].process );
+            m_scripts.erase( *it );
         }
-        m_scripts.erase( *itKeys );
     }
 }
 
 
 bool
-ScriptManager::slotRunScript()
+ScriptManager::slotRunScript( bool silent )
 {
-    DEBUG_BLOCK
+    if( !m_gui->runButton->isEnabled() ) return false;
 
     QListViewItem* const li = m_gui->listView->currentItem();
     const QString name = li->text( 0 );
 
+    if( m_scripts[name].type == "lyrics" && lyricsScriptRunning() != QString::null ) {
+        if( !silent )
+            KMessageBox::sorry( 0, i18n( "Another lyrics script is already running. "
+                                         "You may only run one lyrics script at a time." ) );
+        return false;
+    }
+
+    if( m_scripts[name].type == "transcode" && transcodeScriptRunning() != QString::null ) {
+        if( !silent )
+            KMessageBox::sorry( 0, i18n( "Another transcode script is already running. "
+                                         "You may only run one transcode script at a time." ) );
+        return false;
+    }
+
     // Don't start a script twice
     if( m_scripts[name].process ) return false;
 
-    AmaroKProcIO* script = new AmaroKProcIO();
-    script->setComm( (KProcess::Communication) ( KProcess::All ) );
+    Amarok::ProcIO* script = new Amarok::ProcIO();
+    script->setComm( static_cast<KProcess::Communication>( KProcess::All ) );
     const KURL url = m_scripts[name].url;
     *script << url.path();
-    script->setWorkingDirectory( amaroK::saveLocation( "scripts-data/" ) );
+    script->setWorkingDirectory( Amarok::saveLocation( "scripts-data/" ) );
 
     connect( script, SIGNAL( receivedStderr( KProcess*, char*, int ) ), SLOT( slotReceivedStderr( KProcess*, char*, int ) ) );
     connect( script, SIGNAL( receivedStdout( KProcess*, char*, int ) ), SLOT( slotReceivedStdout( KProcess*, char*, int ) ) );
     connect( script, SIGNAL( processExited( KProcess* ) ), SLOT( scriptFinished( KProcess* ) ) );
 
-    if( !script->start( KProcess::NotifyOnExit ) ) {
-        KMessageBox::sorry( 0, i18n( "<p>Could not start the script <i>%1</i>.</p>"
-                                     "<p>Please make sure that the file has execute (+x) permissions.</p>" ).arg( name ) );
+    if( script->start( KProcess::NotifyOnExit ) )
+    {
+        if( m_scripts[name].type == "score" && !scoreScriptRunning().isNull() )
+        {
+            stopScript( scoreScriptRunning() );
+            m_gui->listView->setCurrentItem( li );
+        }
+        AmarokConfig::setLastScoreScript( name );
+    }
+    else
+    {
+        if( !silent )
+            KMessageBox::sorry( 0, i18n( "<p>Could not start the script <i>%1</i>.</p>"
+                                         "<p>Please make sure that the file has execute (+x) permissions.</p>" ).arg( name ) );
         delete script;
         return false;
     }
 
-    li->setPixmap( 0, SmallIcon( "player_play" ) );
+    li->setPixmap( 0, SmallIcon( Amarok::icon( "play" ) ) );
     debug() << "Running script: " << url.path() << endl;
 
     m_scripts[name].process = script;
     slotCurrentChanged( m_gui->listView->currentItem() );
+    if( m_scripts[name].type == "lyrics" )
+        emit lyricsScriptChanged();
+
     return true;
 }
 
@@ -493,8 +630,6 @@ ScriptManager::slotRunScript()
 void
 ScriptManager::slotStopScript()
 {
-    DEBUG_BLOCK
-
     QListViewItem* const li = m_gui->listView->currentItem();
     const QString name = li->text( 0 );
 
@@ -502,24 +637,17 @@ ScriptManager::slotStopScript()
     if( m_scripts.find( name ) == m_scripts.end() )
         return;
 
-    // Terminate script process (with SIGTERM)
-    m_scripts[name].process->kill();
-    m_scripts[name].process->detach();
-
-    delete m_scripts[name].process;
-    m_scripts[name].process = 0;
+    terminateProcess( &m_scripts[name].process );
     m_scripts[name].log = QString::null;
     slotCurrentChanged( m_gui->listView->currentItem() );
 
-    li->setPixmap( 0, SmallIcon( "stop" ) );
+    li->setPixmap( 0, QPixmap() );
 }
 
 
 void
 ScriptManager::slotConfigureScript()
 {
-    DEBUG_BLOCK
-
     const QString name = m_gui->listView->currentItem()->text( 0 );
     if( !m_scripts[name].process ) return;
 
@@ -527,16 +655,12 @@ ScriptManager::slotConfigureScript()
     QDir::setCurrent( url.directory() );
 
     m_scripts[name].process->writeStdin( "configure" );
-
-    debug() << "Starting script configuration." << endl;
 }
 
 
 void
 ScriptManager::slotAboutScript()
 {
-    DEBUG_BLOCK
-
     const QString name = m_gui->listView->currentItem()->text( 0 );
     QFile readme( m_scripts[name].url.directory( false ) + "README" );
     QFile license( m_scripts[name].url.directory( false ) + "COPYING" );
@@ -554,7 +678,7 @@ ScriptManager::slotAboutScript()
     about->setProduct( "", "", "", "" );
     // Get rid of the confusing KDE version text
     QLabel* product = static_cast<QLabel*>( about->mainWidget()->child( "version" ) );
-    if( product ) product->setText( i18n( "%1 amaroK Script" ).arg( name ) );
+    if( product ) product->setText( i18n( "%1 Amarok Script" ).arg( name ) );
 
     about->addTextPage( i18n( "About" ), readme.readAll(), true );
     if( license.open( IO_ReadOnly ) )
@@ -568,7 +692,12 @@ ScriptManager::slotAboutScript()
 void
 ScriptManager::slotShowContextMenu( QListViewItem* item, const QPoint& pos )
 {
-    if( !item ) return;
+    const bool isCategory = item == m_generalCategory ||
+                            item == m_lyricsCategory ||
+                            item == m_scoreCategory ||
+                            item == m_transcodeCategory;
+
+    if( !item || isCategory ) return;
 
     // Look up script entry in our map
     ScriptMap::Iterator it;
@@ -579,15 +708,15 @@ ScriptManager::slotShowContextMenu( QListViewItem* item, const QPoint& pos )
     enum { SHOW_LOG, EDIT };
     KPopupMenu menu;
     menu.insertTitle( i18n( "Debugging" ) );
-    menu.insertItem( SmallIconSet( "history" ), i18n( "Show Output &Log" ), SHOW_LOG );
-    menu.insertItem( SmallIconSet( "edit" ), i18n( "&Edit" ), EDIT );
+    menu.insertItem( SmallIconSet( Amarok::icon( "clock" ) ), i18n( "Show Output &Log" ), SHOW_LOG );
+    menu.insertItem( SmallIconSet( Amarok::icon( "edit" ) ), i18n( "&Edit" ), EDIT );
     menu.setItemEnabled( SHOW_LOG, it.data().process );
     const int id = menu.exec( pos );
 
     switch( id )
     {
         case EDIT:
-            KRun::runCommand( "kwrite " + it.data().url.path() );
+            KRun::runCommand( "kwrite " + KProcess::quote(it.data().url.path()) );
             break;
 
         case SHOW_LOG:
@@ -612,19 +741,18 @@ ScriptManager::slotShowContextMenu( QListViewItem* item, const QPoint& pos )
     }
 }
 
-/* This is just a workaround, some scripts crash for some people if stdout is not handled.
-   Anyway, printing out the output of scripts can be usefull. */
+
+/* This is just a workaround, some scripts crash for some people if stdout is not handled. */
 void
-ScriptManager::slotReceivedStdout( KProcess* /*process*/, char* buf, int len )
+ScriptManager::slotReceivedStdout( KProcess*, char* buf, int len )
 {
     debug() << QString::fromLatin1( buf, len ) << endl;
 }
 
+
 void
 ScriptManager::slotReceivedStderr( KProcess* process, char* buf, int len )
 {
-    DEBUG_BLOCK
-
     // Look up script entry in our map
     ScriptMap::Iterator it;
     ScriptMap::Iterator end( m_scripts.end() );
@@ -643,8 +771,6 @@ ScriptManager::slotReceivedStderr( KProcess* process, char* buf, int len )
 void
 ScriptManager::scriptFinished( KProcess* process ) //SLOT
 {
-    DEBUG_BLOCK
-
     // Look up script entry in our map
     ScriptMap::Iterator it;
     ScriptMap::Iterator end( m_scripts.end() );
@@ -661,7 +787,7 @@ ScriptManager::scriptFinished( KProcess* process ) //SLOT
     delete it.data().process;
     it.data().process = 0;
     it.data().log = QString::null;
-    it.data().li->setPixmap( 0, SmallIcon( "stop" ) );
+    it.data().li->setPixmap( 0, QPixmap() );
     slotCurrentChanged( m_gui->listView->currentItem() );
 }
 
@@ -670,16 +796,70 @@ ScriptManager::scriptFinished( KProcess* process ) //SLOT
 // private
 ////////////////////////////////////////////////////////////////////////////////
 
+QStringList
+ScriptManager::scriptsOfType( const QString &type ) const
+{
+    QStringList scripts;
+    foreachType( ScriptMap, m_scripts )
+        if( it.data().type == type )
+            scripts += it.key();
+
+    return scripts;
+}
+
+
+QString
+ScriptManager::scriptRunningOfType( const QString &type ) const
+{
+    foreachType( ScriptMap, m_scripts )
+        if( it.data().process )
+            if( it.data().type == type )
+                return it.key();
+
+    return QString();
+}
+
+
+QString
+ScriptManager::ensureScoreScriptRunning()
+{
+    QString s = scoreScriptRunning();
+    if( !s.isNull() )
+        return s;
+
+    if( runScript( AmarokConfig::lastScoreScript(), true /*silent*/ ) )
+        return AmarokConfig::lastScoreScript();
+
+    const QString def = i18n( "Score" ) + ": " + "Default";
+    if( runScript( def, true ) )
+        return def;
+
+    const QStringList scripts = scoreScripts();
+    for( QStringList::const_iterator it = scripts.begin(), end = scripts.end(); it != end; ++it )
+        if( runScript( *it, true ) )
+            return *it;
+
+    return QString();
+}
+
+
+void
+ScriptManager::terminateProcess( KProcIO** proc )
+{
+    if( *proc ) {
+        (*proc)->kill(); // Sends SIGTERM
+        (*proc)->detach();
+
+        delete *proc;
+        *proc = 0;
+    }
+}
+
+
 void
 ScriptManager::notifyScripts( const QString& message )
 {
-    DEBUG_BLOCK
-
-    debug() << "Sending notification: " << message << endl;
-
-    ScriptMap::Iterator it;
-    ScriptMap::Iterator end( m_scripts.end() );
-    for( it = m_scripts.begin(); it != end; ++it ) {
+    foreachType( ScriptMap, m_scripts ) {
         KProcIO* const proc = it.data().process;
         if( proc ) proc->writeStdin( message );
     }
@@ -689,21 +869,43 @@ ScriptManager::notifyScripts( const QString& message )
 void
 ScriptManager::loadScript( const QString& path )
 {
-    DEBUG_BLOCK
-
     if( !path.isEmpty() ) {
         const KURL url = KURL::fromPathOrURL( path );
+        QString name = url.fileName();
+        QString type = "generic";
 
-        KListViewItem* li = new KListViewItem( m_gui->listView, url.fileName() );
-        li->setPixmap( 0, SmallIcon( "stop" ) );
+        // Read and parse .spec file, if exists
+        QFileInfo info( path );
+        KListViewItem* li = 0;
+        const QString specPath = info.dirPath() + '/' + info.baseName( true ) + ".spec";
+        if( QFile::exists( specPath ) ) {
+            KConfig spec( specPath, true, false );
+            if( spec.hasKey( "name" ) )
+                name = spec.readEntry( "name" );
+            if( spec.hasKey( "type" ) ) {
+                type = spec.readEntry( "type" );
+                if( type == "lyrics" )
+                    li = new KListViewItem( m_lyricsCategory, name );
+                if( type == "transcode" )
+                    li = new KListViewItem( m_transcodeCategory, name );
+                if( type == "score" )
+                    li = new KListViewItem( m_scoreCategory, name );
+            }
+        }
+
+        if( !li )
+            li = new KListViewItem( m_generalCategory, name );
+
+        li->setPixmap( 0, QPixmap() );
 
         ScriptItem item;
         item.url = url;
+        item.type = type;
         item.process = 0;
         item.li = li;
 
-        m_scripts[url.fileName()] = item;
-        debug() << "Loaded: " << url.fileName() << endl;
+        m_scripts[name] = item;
+        debug() << "Loaded: " << name << endl;
 
         slotCurrentChanged( m_gui->listView->currentItem() );
     }
@@ -713,8 +915,6 @@ ScriptManager::loadScript( const QString& path )
 void
 ScriptManager::engineStateChanged( Engine::State state, Engine::State /*oldState*/ )
 {
-    DEBUG_BLOCK
-
     switch( state )
     {
         case Engine::Empty:
@@ -749,5 +949,4 @@ ScriptManager::engineVolumeChanged( int newVolume )
     notifyScripts( "volumeChange: " + QString::number( newVolume ) );
 }
 
-
 #include "scriptmanager.moc"
