@@ -22,9 +22,10 @@
 
 #include "amarok.h"
 #include "debug.h"
-#include "overlayWidget.h"
 #include "squeezedtextlabel.h"
 #include "statusBarBase.h"
+#include "threadmanager.h"
+#include "enginecontroller.h"
 
 #include <kio/job.h>
 #include <kiconloader.h>
@@ -60,10 +61,8 @@ namespace SingleShotPool
 {
     static void startTimer( int timeout, QObject *receiver, const char *slot )
     {
-        QTimer *timer = (QTimer*)receiver->child( slot );
+        QTimer *timer = static_cast<QTimer*>( receiver->child( slot ) );
         if( !timer ) {
-            debug() << "Creating timer for: " << slot << endl;
-
             timer = new QTimer( receiver, slot );
             receiver->connect( timer, SIGNAL(timeout()), slot );
         }
@@ -73,7 +72,7 @@ namespace SingleShotPool
 
     static inline bool isActive( QObject *parent, const char *slot )
     {
-        QTimer *timer = (QTimer*)parent->child( slot );
+        QTimer *timer = static_cast<QTimer*>( parent->child( slot ) );
 
         return timer && timer->isA( "QTimer" ) && timer->isActive();
     }
@@ -92,7 +91,10 @@ StatusBar::StatusBar( QWidget *parent, const char *name )
     //we need extra spacing due to the way we paint the surrounding boxes
     QBoxLayout *layout = new QHBoxLayout( mainlayout, /*spacing*/5 );
 
-    m_mainTextLabel = new KDE::SqueezedTextLabel( this, "mainTextLabel" );
+    QHBox *statusBarTextBox = new QHBox( this, "statusBarTextBox" );
+    m_mainTextLabel = new KDE::SqueezedTextLabel( statusBarTextBox, "mainTextLabel" );
+    QToolButton *shortLongButton = new QToolButton( statusBarTextBox, "shortLongButton" );
+    shortLongButton->hide();
 
     QHBox *mainProgressBarBox = new QHBox( this, "progressBox" );
     QToolButton *b1 = new QToolButton( mainProgressBarBox, "cancelButton" );
@@ -101,9 +103,9 @@ StatusBar::StatusBar( QWidget *parent, const char *name )
     mainProgressBarBox->setSpacing( 2 );
     mainProgressBarBox->hide();
 
-    layout->add( m_mainTextLabel );
+    layout->add( statusBarTextBox );
     layout->add( mainProgressBarBox );
-    layout->setStretchFactor( m_mainTextLabel, 3 );
+    layout->setStretchFactor( statusBarTextBox, 3 );
     layout->setStretchFactor( mainProgressBarBox, 1 );
 
     m_otherWidgetLayout = new QHBoxLayout( mainlayout, /*spacing*/5 );
@@ -111,6 +113,10 @@ StatusBar::StatusBar( QWidget *parent, const char *name )
     mainlayout->setStretchFactor( layout, 6 );
     mainlayout->setStretchFactor( m_otherWidgetLayout, 4 );
 
+    shortLongButton->setIconSet( SmallIconSet( "edit_add" ) );
+    QToolTip::add( shortLongButton, i18n( "Show details" ) );
+    connect( shortLongButton, SIGNAL(clicked()), SLOT(showShortLongDetails()) );
+
     b1->setIconSet( SmallIconSet( "cancel" ) );
     b2->setIconSet( SmallIconSet( "2uparrow") );
     b2->setToggleButton( true );
@@ -121,7 +127,8 @@ StatusBar::StatusBar( QWidget *parent, const char *name )
 
     m_popupProgress = new OverlayWidget( this, mainProgressBarBox, "popupProgress" );
     m_popupProgress->setMargin( 1 );
-    m_popupProgress->setFrameStyle( QFrame::Box | QFrame::Raised );
+    m_popupProgress->setFrameStyle( QFrame::Panel | QFrame::Raised );
+    m_popupProgress->setFrameShape( QFrame::StyledPanel );
     m_popupProgress->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
    (new QGridLayout( m_popupProgress, 1 /*rows*/, 3 /*cols*/, 6, 3 ))->setAutoAdd( true );
 }
@@ -148,7 +155,7 @@ StatusBar::polish()
         if ( _h > h )
             h = _h;
 
-        debug() << o->className() << ", " << o->name() << ": " << _h << ": " << static_cast<QWidget*>(o)->minimumHeight() << endl;
+//         debug() << o->className() << ", " << o->name() << ": " << _h << ": " << static_cast<QWidget*>(o)->minimumHeight() << endl;
 
         if ( o->inherits( "QLabel" ) )
             static_cast<QLabel*>(o)->setIndent( 4 );
@@ -169,7 +176,7 @@ StatusBar::paintEvent( QPaintEvent* )
     QPainter p( this );
 
     for( QObject * o = list->first(); o; o = list->next() ) {
-        QWidget *w = (QWidget*)o;
+        QWidget *w = static_cast<QWidget*>( o );
 
         if ( !w->isVisible() )
             continue;
@@ -201,6 +208,8 @@ StatusBar::event( QEvent *e )
 void
 StatusBar::setMainText( const QString &text )
 {
+    SHOULD_BE_GUI
+
     m_mainText = text;
 
     // it may not be appropriate for us to set the mainText yet
@@ -208,12 +217,14 @@ StatusBar::setMainText( const QString &text )
 }
 
 void
-StatusBar::shortMessage( const QString &text )
+StatusBar::shortMessage( const QString &text, bool longShort )
 {
+    SHOULD_BE_GUI
+
     m_mainTextLabel->setText( text );
     m_mainTextLabel->setPalette( QToolTip::palette() );
 
-    SingleShotPool::startTimer( 5000, this, SLOT(resetMainText()) );
+    SingleShotPool::startTimer( longShort ? 8000 : 5000, this, SLOT(resetMainText()) );
 
     writeLogFile( text );
 }
@@ -221,21 +232,20 @@ StatusBar::shortMessage( const QString &text )
 void
 StatusBar::resetMainText()
 {
-    if( sender() )
-        debug() << sender()->name() << endl;
+//     if( sender() )
+//         debug() << sender()->name() << endl;
 
     // don't reset if we are showing a shortMessage
     if( SingleShotPool::isActive( this, SLOT(resetMainText()) ) )
         return;
 
     m_mainTextLabel->unsetPalette();
+    shortLongButton()->hide();
 
     if( allDone() )
         m_mainTextLabel->setText( m_mainText );
 
     else {
-        m_mainTextLabel->setPaletteBackgroundColor( m_mainTextLabel->paletteBackgroundColor().dark( 110 ) );
-
         ProgressBar *bar = 0;
         uint count = 0;
         foreachType( ProgressMap, m_progressMap )
@@ -245,30 +255,40 @@ StatusBar::resetMainText()
             }
 
         if( count == 1 )
-            m_mainTextLabel->setText( bar->description() + "..." );
+            m_mainTextLabel->setText( bar->description() + i18n("...") );
         else
             m_mainTextLabel->setText( i18n("Multiple background-tasks running") );
     }
 }
 
 void
-StatusBar::shortLongMessage( const QString &_short, const QString &_long )
+StatusBar::shortLongMessage( const QString &_short, const QString &_long, int type )
 {
-    shortMessage( _short );
+    SHOULD_BE_GUI
+
+    m_shortLongType = type;
+
+    if( !_short.isEmpty() )
+        shortMessage( _short, true );
 
     if ( !_long.isEmpty() ) {
-        AMAROK_NOTIMPLEMENTED
+        m_shortLongText = _long;
+        shortLongButton()->show();
+        writeLogFile( _long );
     }
 }
 
 void
 StatusBar::longMessage( const QString &text, int type )
 {
+    SHOULD_BE_GUI
+
     if( text.isEmpty() )
         return;
 
-    PopupMessage * message;
+    PopupMessage *message;
     message = new PopupMessage( this, m_mainTextLabel );
+    connect( message, SIGNAL(destroyed(QObject *)), this, SLOT(popupDeleted(QObject *)) );
     message->setText( text );
 
     QString image;
@@ -287,15 +307,17 @@ StatusBar::longMessage( const QString &text, int type )
 
         case Error:
             image = KGlobal::iconLoader()->iconPath( "messagebox_critical", -KIcon::SizeHuge );
+            // don't hide error messages.
+//             message->setTimeout( 0 );
             break;
     }
+
     if( !image.isEmpty() )
         message->setImage( image );
 
     if ( !m_messageQueue.isEmpty() )
          message->stackUnder( m_messageQueue.last() );
 
-    message->reposition();
     message->display();
 
     raise();
@@ -305,6 +327,12 @@ StatusBar::longMessage( const QString &text, int type )
     writeLogFile( text );
 }
 
+void
+StatusBar::popupDeleted( QObject *obj )
+{
+    m_messageQueue.remove( static_cast<QWidget*>( obj ) );
+}
+
 void
 StatusBar::longMessageThreadSafe( const QString &text, int /*type*/ )
 {
@@ -316,9 +344,13 @@ StatusBar::longMessageThreadSafe( const QString &text, int /*type*/ )
 void
 StatusBar::customEvent( QCustomEvent *e )
 {
-    QString *s = static_cast<QString*>( e->data() );
-    longMessage( *s );
-    delete s;
+    if(e->type() == 1000 ){ 
+      QString *s = static_cast<QString*>( e->data() );
+      longMessage( *s );
+      delete s;
+    }else if(e->type() == 2000 ){
+      EngineController::instance()->unplayableNotification();
+    }
 }
 
 
@@ -337,6 +369,8 @@ StatusBar::allDone()
 ProgressBar&
 StatusBar::newProgressOperation( QObject *owner )
 {
+    SHOULD_BE_GUI
+
     if ( m_progressMap.contains( owner ) )
         return *m_progressMap[owner];
 
@@ -345,10 +379,11 @@ StatusBar::newProgressOperation( QObject *owner )
         // we start anything new or the total progress will not be accurate
         pruneProgressBars();
     else
-        static_cast<QWidget*>(progressBox()->child("showAllProgressDetails"))->show();
+        toggleProgressWindowButton()->show();
     QLabel *label = new QLabel( m_popupProgress );
     m_progressMap.insert( owner, new ProgressBar( m_popupProgress, label ) );
 
+    m_popupProgress->reposition();
 
     connect( owner, SIGNAL(destroyed( QObject* )), SLOT(endProgressOperation( QObject* )) );
 
@@ -365,11 +400,13 @@ StatusBar::newProgressOperation( QObject *owner )
 ProgressBar&
 StatusBar::newProgressOperation( KIO::Job *job )
 {
-    ProgressBar & bar = newProgressOperation( (QObject*)job );
+    SHOULD_BE_GUI
+
+    ProgressBar & bar = newProgressOperation( static_cast<QObject*>( job ) );
     bar.setTotalSteps( 100 );
 
     if(!allDone())
-        static_cast<QWidget*>(progressBox()->child("showAllProgressDetails"))->show();
+        toggleProgressWindowButton()->show();
     connect( job, SIGNAL(result( KIO::Job* )), SLOT(endProgressOperation()) );
     //TODO connect( job, SIGNAL(infoMessage( KIO::Job *job, const QString& )), SLOT() );
     connect( job, SIGNAL(percent( KIO::Job*, unsigned long )), SLOT(setProgress( KIO::Job*, unsigned long )) );
@@ -380,14 +417,14 @@ StatusBar::newProgressOperation( KIO::Job *job )
 void
 StatusBar::endProgressOperation()
 {
-    QObject *owner = (QObject*)sender(); //HACK deconsting it
+    QObject *owner = const_cast<QObject*>( sender() ); //HACK deconsting it
     KIO::Job *job = dynamic_cast<KIO::Job*>( owner );
 
     //FIXME doesn't seem to work for KIO::DeleteJob, it has it's own error handler and returns no error too
     // if you try to delete http urls for instance <- KDE SUCKS!
 
     if( job && job->error() )
-        longMessage( job->errorString(), Error );
+        shortLongMessage( QString::null, job->errorString(), Error );
 
     endProgressOperation( owner );
 }
@@ -401,7 +438,10 @@ StatusBar::endProgressOperation( QObject *owner )
     //things, we just call setDone().
 
     if ( !m_progressMap.contains( owner ) )
+    {
+        SingleShotPool::startTimer( 2000, this, SLOT(hideMainProgressBar()) );
         return ;
+    }
 
     m_progressMap[owner]->setDone();
 
@@ -427,13 +467,24 @@ StatusBar::abortAllProgressOperations() //slot
 void
 StatusBar::toggleProgressWindow( bool show ) //slot
 {
-    m_popupProgress->adjustSize(); //FIXME shouldn't be needed, adding bars doesn't seem to do this
+    m_popupProgress->reposition(); //FIXME shouldn't be needed, adding bars doesn't seem to do this
     m_popupProgress->setShown( show );
 
     if( !show )
         SingleShotPool::startTimer( 2000, this, SLOT(hideMainProgressBar()) );
 }
 
+void
+StatusBar::showShortLongDetails()
+{
+    if( !m_shortLongText.isEmpty() )
+        longMessage( m_shortLongText, m_shortLongType );
+
+    m_shortLongType = Information;
+    m_shortLongText = QString::null;
+    shortLongButton()->hide();
+}
+
 void
 StatusBar::showMainProgressBar()
 {
@@ -464,7 +515,7 @@ StatusBar::setProgress( int steps )
 void
 StatusBar::setProgress( KIO::Job *job, unsigned long percent )
 {
-    setProgress( ( QObject* ) job, percent );
+    setProgress( static_cast<QObject*>( job ), percent );
 }
 
 void
@@ -478,6 +529,17 @@ StatusBar::setProgress( const QObject *owner, int steps )
     updateTotalProgress();
 }
 
+void
+StatusBar::incrementProgressTotalSteps( const QObject *owner, int inc )
+{
+    if ( !m_progressMap.contains( owner ) )
+        return ;
+
+    m_progressMap[ owner ] ->setTotalSteps( m_progressMap[ owner ] ->totalSteps() + inc );
+
+    updateTotalProgress();
+}
+
 void
 StatusBar::setProgressStatus( const QObject *owner, const QString &text )
 {
@@ -558,7 +620,7 @@ StatusBar::pruneProgressBars()
     if(count==1 && removedBar) //if its gone from 2 or more bars to one bar...
     {
         resetMainText();
-        static_cast<QWidget*>(progressBox()->child("showAllProgressDetails"))->hide();
+        toggleProgressWindowButton()->hide();
         m_popupProgress->setShown(false);
     }
 }
@@ -570,8 +632,9 @@ StatusBar::writeLogFile( const QString &text )
     if( text.isEmpty() ) return;
 
     const int counter = 4; // number of logs to keep
+    const uint maxSize = 30000; // approximately 1000 lines per log file
     int c = counter;
-    QString logBase = amaroK::saveLocation() + "statusbar.log.";
+    QString logBase = Amarok::saveLocation() + "statusbar.log.";
     QFile file;
 
     if( m_logCounter < 0 ) //find which log to write to
@@ -579,13 +642,12 @@ StatusBar::writeLogFile( const QString &text )
         for( ; c > 0; c-- )
         {
             QString log = logBase + QString::number(c);
+            file.setName( log );
 
-            if( QFile::exists( log ) )
-            {
-                file.setName( log );
-            }
+            if( QFile::exists( log ) && file.size() <= maxSize )
+                break;
         }
-        if( c == 0 ) file.setName( logBase + "0" );
+        if( c == 0 ) file.setName( logBase + '0' );
         m_logCounter = c;
     }
     else
@@ -593,7 +655,7 @@ StatusBar::writeLogFile( const QString &text )
         file.setName( logBase + QString::number(m_logCounter) );
     }
 
-    if( file.size() > 30000 ) // approximately 1000 lines per log file
+    if( file.size() > maxSize )
     {
         m_logCounter++;
         m_logCounter = m_logCounter % counter;
@@ -607,7 +669,7 @@ StatusBar::writeLogFile( const QString &text )
     QTextStream stream( &file );
     stream.setEncoding( QTextStream::UnicodeUTF8 );
 
-    stream << "[" << QDateTime::currentDateTime().toString(Qt::LocalDate) << "] " << text << endl;
+    stream << "[" << KGlobal::locale()->formatDateTime( QDateTime::currentDateTime() ) << "] " << text << endl;
 }
 
 } //namespace KDE
