@@ -2,9 +2,9 @@
 // AmarokSystray
 //
 // Contributors: Stanislav Karchebny <berkus@users.sf.net>, (C) 2003
-//               berkus, mxcl, eros
+//               berkus, mxcl, eros, eean
 //
-// Copyright: like rest of amaroK
+// Copyright: like rest of Amarok
 //
 
 #include "amarok.h"
@@ -20,7 +20,7 @@
 #include <kiconeffect.h>
 #include <kstandarddirs.h>
 
-namespace amaroK
+namespace Amarok
 {
     static QPixmap
     loadOverlay( const char *iconName )
@@ -30,7 +30,7 @@ namespace amaroK
 }
 
 
-amaroK::TrayIcon::TrayIcon( QWidget *playerWidget )
+Amarok::TrayIcon::TrayIcon( QWidget *playerWidget )
         : KSystemTray( playerWidget )
         , EngineObserver( EngineController::instance() )
         , trackLength( 0 )
@@ -38,16 +38,16 @@ amaroK::TrayIcon::TrayIcon( QWidget *playerWidget )
         , overlay( 0 )
         , blinkTimerID( 0 )
         , overlayVisible( false )
+        , m_lastFmMode( false )
 {
-    KActionCollection* const ac = amaroK::actionCollection();
+    KActionCollection* const ac = Amarok::actionCollection();
 
     setAcceptDrops( true );
 
-    ac->action( "prev"  )->plug( contextMenu() );
-    ac->action( "play"  )->plug( contextMenu() );
-    ac->action( "pause" )->plug( contextMenu() );
-    ac->action( "stop"  )->plug( contextMenu() );
-    ac->action( "next"  )->plug( contextMenu() );
+    ac->action( "prev"        )->plug( contextMenu() );
+    ac->action( "play_pause"  )->plug( contextMenu() );
+    ac->action( "stop"        )->plug( contextMenu() );
+    ac->action( "next"        )->plug( contextMenu() );
 
     //seems to be necessary
     KAction *quit = actionCollection()->action( "file_quit" );
@@ -55,8 +55,8 @@ amaroK::TrayIcon::TrayIcon( QWidget *playerWidget )
     connect( quit, SIGNAL(activated()), kapp, SLOT(quit()) );
 
     baseIcon     = KSystemTray::loadIcon( "amarok" );
-    playOverlay  = amaroK::loadOverlay( "play" );
-    pauseOverlay = amaroK::loadOverlay( "pause" );
+    playOverlay  = Amarok::loadOverlay( "play" );
+    pauseOverlay = Amarok::loadOverlay( "pause" );
     overlayVisible = false;
 
     //paintIcon();
@@ -64,14 +64,14 @@ amaroK::TrayIcon::TrayIcon( QWidget *playerWidget )
 }
 
 bool
-amaroK::TrayIcon::event( QEvent *e )
+Amarok::TrayIcon::event( QEvent *e )
 {
     switch( e->type() )
     {
     case QEvent::Drop:
     case QEvent::Wheel:
     case QEvent::DragEnter:
-        return amaroK::genericEventHandler( this, e );
+        return Amarok::genericEventHandler( this, e );
 
     case QEvent::Timer:
         if( static_cast<QTimerEvent*>(e)->timerId() != blinkTimerID )
@@ -102,7 +102,7 @@ amaroK::TrayIcon::event( QEvent *e )
 }
 
 void
-amaroK::TrayIcon::engineStateChanged( Engine::State state, Engine::State /*oldState*/ )
+Amarok::TrayIcon::engineStateChanged( Engine::State state, Engine::State /*oldState*/ )
 {
     // stop timer
     if ( blinkTimerID )
@@ -132,27 +132,28 @@ amaroK::TrayIcon::engineStateChanged( Engine::State state, Engine::State /*oldSt
     case Engine::Empty:
         overlayVisible = false;
         paintIcon( -1, true ); // repaint the icon
-
+                               // fall through to default:
     default:
-        ;
+        setLastFm( false );
     }
 }
 
 void
-amaroK::TrayIcon::engineNewMetaData( const MetaBundle &bundle, bool /*trackChanged*/ )
+Amarok::TrayIcon::engineNewMetaData( const MetaBundle &bundle, bool /*trackChanged*/ )
 {
     trackLength = bundle.length() * 1000;
+    setLastFm( bundle.url().protocol() == "lastfm" );
 }
 
 void
-amaroK::TrayIcon::engineTrackPositionChanged( long position, bool /*userSeek*/ )
+Amarok::TrayIcon::engineTrackPositionChanged( long position, bool /*userSeek*/ )
 {
     mergeLevel = trackLength ? ((baseIcon.height() + 1) * position) / trackLength : -1;
     paintIcon( mergeLevel );
 }
 
 void
-amaroK::TrayIcon::paletteChange( const QPalette & op )
+Amarok::TrayIcon::paletteChange( const QPalette & op )
 {
     if ( palette().active().highlight() == op.active().highlight() || alternateIcon.isNull() )
         return;
@@ -162,7 +163,7 @@ amaroK::TrayIcon::paletteChange( const QPalette & op )
 }
 
 void
-amaroK::TrayIcon::paintIcon( int mergePixels, bool force )
+Amarok::TrayIcon::paintIcon( int mergePixels, bool force )
 {
     // skip redrawing the same pixmap
     static int mergePixelsCache = 0;
@@ -180,8 +181,6 @@ amaroK::TrayIcon::paintIcon( int mergePixels, bool force )
         KIconEffect::semiTransparent( tmpTrayIcon );
         grayedIcon = tmpTrayIcon;
     }
-    if ( mergePixels == 0 )
-        return blendOverlay( grayedIcon );
 
     // make up the alternate icon (use hilight color but more saturated)
     if ( alternateIcon.isNull() )
@@ -197,18 +196,21 @@ amaroK::TrayIcon::paintIcon( int mergePixels, bool force )
         KIconEffect::colorize( tmpTrayIcon, saturatedColor/* Qt::blue */, 0.9 );
         alternateIcon = tmpTrayIcon;
     }
+
     if ( mergePixels >= alternateIcon.height() )
+        return blendOverlay( grayedIcon );
+    if ( mergePixels == 0 )
         return blendOverlay( alternateIcon );
 
     // mix [ grayed <-> colored ] icons
     QPixmap tmpTrayPixmap = alternateIcon;
     copyBlt( &tmpTrayPixmap, 0,0, &grayedIcon, 0,0,
-            alternateIcon.width(), alternateIcon.height() - mergePixels );
+            alternateIcon.width(), mergePixels>0 ? mergePixels-1 : 0 );
     blendOverlay( tmpTrayPixmap );
 }
 
 void
-amaroK::TrayIcon::blendOverlay( QPixmap &sourcePixmap )
+Amarok::TrayIcon::blendOverlay( QPixmap &sourcePixmap )
 {
     if ( !overlayVisible || !overlay || overlay->isNull() )
         return setPixmap( sourcePixmap ); // @since 3.2
@@ -242,3 +244,37 @@ amaroK::TrayIcon::blendOverlay( QPixmap &sourcePixmap )
 
     setPixmap( sourcePixmapCopy ); // @since 3.2
 }
+
+void
+Amarok::TrayIcon::setLastFm( bool lastFmActive )
+{
+    if( lastFmActive == m_lastFmMode ) return;
+
+    static int separatorId = 0;
+
+    KActionCollection* const ac = Amarok::actionCollection();
+    if( ac->action( "ban" ) == 0 ) return; //if the LastFm::Controller doesn't exist yet
+
+    if( lastFmActive )
+    {
+        ac->action( "play_pause" )->unplug( contextMenu() );
+        // items are inserted in reverse order!
+        ac->action( "ban" ) ->plug( contextMenu(), 4 );
+        ac->action( "love" )->plug( contextMenu(), 4 );
+        ac->action( "skip" )->plug( contextMenu(), 4 );
+        separatorId = contextMenu()->insertSeparator( 4 );
+
+        m_lastFmMode = true;
+    }
+    else
+    {
+        ac->action( "play_pause" )->plug( contextMenu(), 2 );
+        ac->action( "ban" ) ->unplug( contextMenu() );
+        ac->action( "love" )->unplug( contextMenu() );
+        ac->action( "skip" )->unplug( contextMenu() );
+
+        if( separatorId != 0 )
+            contextMenu()->removeItem( separatorId ); // kill separator
+        m_lastFmMode = false;
+   }
+}
